@@ -1,28 +1,14 @@
-// Copyright(c) 2017-2019 Alejandro Sirgo Rica & Contributors
-//
-// This file is part of Flameshot.
-//
-//     Flameshot is free software: you can redistribute it and/or modify
-//     it under the terms of the GNU General Public License as published by
-//     the Free Software Foundation, either version 3 of the License, or
-//     (at your option) any later version.
-//
-//     Flameshot is distributed in the hope that it will be useful,
-//     but WITHOUT ANY WARRANTY; without even the implied warranty of
-//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-//     GNU General Public License for more details.
-//
-//     You should have received a copy of the GNU General Public License
-//     along with Flameshot.  If not, see <http://www.gnu.org/licenses/>.
+// SPDX-License-Identifier: GPL-3.0-or-later
+// SPDX-FileCopyrightText: 2017-2019 Alejandro Sirgo Rica & Contributors
 
 #include "visualseditor.h"
 #include "src/config/buttonlistview.h"
+#include "src/config/colorpickereditor.h"
 #include "src/config/extendedslider.h"
 #include "src/config/uicoloreditor.h"
 #include "src/utils/confighandler.h"
 #include <QHBoxLayout>
 #include <QLabel>
-#include <QVBoxLayout>
 
 VisualsEditor::VisualsEditor(QWidget* parent)
   : QWidget(parent)
@@ -36,7 +22,7 @@ void VisualsEditor::updateComponents()
 {
     m_buttonList->updateComponents();
     m_colorEditor->updateComponents();
-    int opacity = ConfigHandler().contrastOpacityValue();
+    int opacity = ConfigHandler().contrastOpacity();
     m_opacitySlider->setMapedValue(0, opacity, 255);
 }
 
@@ -46,49 +32,57 @@ void VisualsEditor::initOpacitySlider()
     m_opacitySlider->setFocusPolicy(Qt::NoFocus);
     m_opacitySlider->setOrientation(Qt::Horizontal);
     m_opacitySlider->setRange(0, 100);
-    connect(m_opacitySlider,
-            &ExtendedSlider::modificationsEnded,
-            this,
-            &VisualsEditor::saveOpacity);
-    QHBoxLayout* localLayout = new QHBoxLayout();
+    auto* localLayout = new QHBoxLayout();
     localLayout->addWidget(new QLabel(QStringLiteral("0%")));
     localLayout->addWidget(m_opacitySlider);
     localLayout->addWidget(new QLabel(QStringLiteral("100%")));
 
-    QLabel* label = new QLabel();
+    auto* label = new QLabel();
     QString labelMsg = tr("Opacity of area outside selection:") + " %1%";
+    ExtendedSlider* opacitySlider = m_opacitySlider;
     connect(m_opacitySlider,
             &ExtendedSlider::valueChanged,
             this,
-            [labelMsg, label](int val) { label->setText(labelMsg.arg(val)); });
+            [labelMsg, label, opacitySlider](int val) {
+                label->setText(labelMsg.arg(val));
+                ConfigHandler().setContrastOpacity(
+                  opacitySlider->mappedValue(0, 255));
+            });
     m_layout->addWidget(label);
     m_layout->addLayout(localLayout);
 
-    int opacity = ConfigHandler().contrastOpacityValue();
+    int opacity = ConfigHandler().contrastOpacity();
     m_opacitySlider->setMapedValue(0, opacity, 255);
 }
 
-void VisualsEditor::saveOpacity()
-{
-    int value = m_opacitySlider->mappedValue(0, 255);
-    ConfigHandler().setContrastOpacity(value);
-}
-
 void VisualsEditor::initWidgets()
 {
+    m_tabWidget = new QTabWidget();
+    m_layout->addWidget(m_tabWidget);
+
     m_colorEditor = new UIcolorEditor();
-    m_layout->addWidget(m_colorEditor);
+    m_colorEditorTab = new QWidget();
+    auto* colorEditorLayout = new QVBoxLayout(m_colorEditorTab);
+    m_colorEditorTab->setLayout(colorEditorLayout);
+    colorEditorLayout->addWidget(m_colorEditor);
+    m_tabWidget->addTab(m_colorEditorTab, tr("UI Color Editor"));
+
+    m_colorpickerEditor = new ColorPickerEditor();
+    m_colorpickerEditorTab = new QWidget();
+    auto* colorpickerEditorLayout = new QVBoxLayout(m_colorpickerEditorTab);
+    colorpickerEditorLayout->addWidget(m_colorpickerEditor);
+    m_tabWidget->addTab(m_colorpickerEditorTab, tr("Colorpicker Editor"));
 
     initOpacitySlider();
 
-    auto boxButtons = new QGroupBox();
+    auto* boxButtons = new QGroupBox();
     boxButtons->setTitle(tr("Button Selection"));
-    auto listLayout = new QVBoxLayout(boxButtons);
+    auto* listLayout = new QVBoxLayout(boxButtons);
     m_buttonList = new ButtonListView();
     m_layout->addWidget(boxButtons);
     listLayout->addWidget(m_buttonList);
 
-    QPushButton* setAllButtons = new QPushButton(tr("Select All"));
+    auto* setAllButtons = new QPushButton(tr("Select All"));
     connect(setAllButtons,
             &QPushButton::clicked,
             m_buttonList,
